@@ -103,6 +103,24 @@ def test_definitions_as_dict():
     assert mcont_dict["CONTOUR_MIN_LEVEL"] == 4.9
 
 
+def test_definitions_as_args():
+    mcont_def = mv.mcont(
+        legend=True,
+        contour_line_thickness=7,
+        contour_level_selection_type="level_list",
+        contour_level_list=[5, 7, 8.8, 9],
+        contour_hi_text="My high text",
+        CONTOUR_MIN_LEVEL=4.9,
+    )
+    mcont_dict = mcont_def.copy()
+    assert mcont_dict["LEGEND"] == "ON"
+    assert mcont_def["CONTOUR_LINE_THICKNESS"] == 7
+    assert mcont_def["CONTOUR_LEVEL_SELECTION_TYPE"] == "LEVEL_LIST"
+    assert mcont_def["CONTOUR_LEVEL_LIST"] == [5, 7, 8.8, 9]
+    assert mcont_def["CONTOUR_HI_TEXT"] == "My high text"
+    assert mcont_def["CONTOUR_MIN_LEVEL"] == 4.9
+
+
 def test_modify_request():
     c = mv.mcont(
         {
@@ -414,19 +432,19 @@ def test_division_fieldsets():
 
 
 def test_power():
-    raised_two = TEST_FIELDSET ** 2
+    raised_two = TEST_FIELDSET**2
     maximum = mv.maxvalue(raised_two)
-    assert np.isclose(maximum, MAX_VALUE ** 2)
+    assert np.isclose(maximum, MAX_VALUE**2)
 
 
 def test_power_reverse():
     mask = TEST_FIELDSET > 290
     FS_3_AND_4 = (mask * 3) + (1 - mask) * 4
-    raised = 2 ** FS_3_AND_4
+    raised = 2**FS_3_AND_4
     minimum = mv.minvalue(raised)
     maximum = mv.maxvalue(raised)
-    assert np.isclose(minimum, 2 ** 3)
-    assert np.isclose(maximum, 2 ** 4)
+    assert np.isclose(minimum, 2**3)
+    assert np.isclose(maximum, 2**4)
 
 
 def test_mod():
@@ -1262,6 +1280,49 @@ def test_geopoints_nonequality_operator():
     assert vdiff[15] == 1
 
 
+def _optional_check_create_geo_inline():
+    # creates geopoints using numpy arrays and lists
+    # values are valid, np.nan and None
+    # - the ability to handle np.nan in lists is only available
+    #   in Metview 5.15.0 and above
+    g = mv.create_geo(
+        type="ncols",
+        latitudes=np.array([4, 5, np.nan]),
+        longitudes=[2.3, np.nan, 6.5],
+        levels=850,  # all rows will have 850 as their level
+        dates=[20180808, 20170707, 20160606],  # dates as numbers
+        times=None,
+        stnids=["aberdeen", "aviemore", "edinburgh"],
+        elevations=np.array([np.nan, 14.1, np.nan]),
+        temp=[273.15, np.nan, 281.45],
+        precip=[None, np.nan, 1],
+        speed=np.array([2, 3, 5]),
+    )
+
+    def check_columns(gpt):
+        aeq = np.testing.assert_array_equal
+        aeq(mv.latitudes(gpt), np.array([4, 5, np.nan]))
+        aeq(mv.longitudes(gpt), np.array([2.3, np.nan, 6.5]))
+        aeq(mv.levels(gpt), np.array([850, 850, 850]))
+        aeq(mv.elevations(gpt), np.array([np.nan, 14.1, np.nan]))
+        aeq(gpt["temp"], np.array([273.15, np.nan, 281.45]))
+        aeq(gpt["precip"], np.array([np.nan, np.nan, 1]))
+
+    check_columns(g)
+    temp_file = file_in_testdir("created_geo.gpts")
+    # check that it's written to disk ok
+    g.write(temp_file)
+    h = mv.read(temp_file)
+    check_columns(h)
+    os.remove(temp_file)
+
+
+def test_create_geo_inline():
+    version = mv.version_info()
+    if version["metview_version"] >= 51500.0:
+        _optional_check_create_geo_inline()
+
+
 def test_geopoints_set_dates():
     g = mv.create_geo(3)
 
@@ -2371,3 +2432,34 @@ def test_download_gallery_data_bad_fname():
     fname = "zzz_for_spectra.grib"
     with pytest.raises(Exception):
         g = mv.gallery.load_dataset(fname)
+
+
+def test_speed():
+    # test with grib written with write() function
+    fs = mv.Fieldset(path=os.path.join(PATH, "tuv_pl.grib"))
+
+    fs_u = fs.select(shortName="u")
+    fs_v = fs.select(shortName="v")
+
+    # single field
+    u = fs_u[0]
+    v = fs_v[0]
+    r = mv.speed(u, v)
+    assert len(r) == 1
+    np.testing.assert_allclose(
+        r.values(), np.sqrt(np.square(u.values()) + np.square(v.values())), rtol=1e-05
+    )
+    assert r.grib_get_long("paramId") == 10
+
+    # multi fields
+    u = fs_u[:2]
+    v = fs_v[:2]
+    r = mv.speed(u, v)
+    assert len(r) == 2
+    for i in range(len(r)):
+        np.testing.assert_allclose(
+            r[i].values(),
+            np.sqrt(np.square(u[i].values()) + np.square(v[i].values())),
+            rtol=1e-05,
+        )
+    assert r.grib_get_long("paramId") == [10, 10]
